module Motion

  class Generate < Thor

    include Thor::Actions

    def self.source_root
      File.join File.dirname(__FILE__), "templates"
    end

    MASTER_COMMAND = "motion-generate".freeze


    # ===========
    # = Actions =
    # ===========

    option :name
    option :parent_class, {
      desc: "The name of the class to inherit from",
      aliases: ["-p", "--parent"],
      default: "UIViewController",
      type: :string,
    }
    desc "ui_view_controller", "Creates a UIViewController subclass in app/controllers"
    def ui_view_controller(name)
      @name = name
      @class_name = "%sController" % @name.camelize
      template "ui_view_controller.rb.erb",
        "app/controllers/#{name.underscorize}_controller.rb"
    end

    option :name
    option :parent_class, {
      desc: "The name of the class to inherit from",
      aliases: ["-p", "--parent"],
      default: "UICollectionViewController",
      type: :string,
    }
    desc "ui_collection_view_controller", "Creates a UICollectionViewController subclass in app/controllers"
    def ui_collection_view_controller(name)
      @name = name
      @class_name = "%sController" % @name.camelize
      template "ui_collection_view_controller.rb.erb",
        "app/controllers/#{name.underscore}_controller.rb"
    end

    option :name
    option :parent_class, {
      desc: "The name of the class to inherit from",
      aliases: ["-p", "--parent"],
      default: "UITableViewController",
      type: :string,
    }
    desc "ui_table_view_controller", "Creates a UITableViewController subclass in app/controllers"
    def ui_table_view_controller(name)
      @name = name
      @class_name = "%sController" % @name.camelize
      template "ui_table_view_controller.rb.erb",
        "app/controllers/#{name.underscorize}_controller.rb"
    end

    option :name
    option :parent_class, {
      desc: "The name of the class to inherit from",
      aliases: ["-p", "--parent"],
      default: "UITableViewCell",
      type: :string,
    }
    desc "ui_table_view_cell", "Creates a UITableViewCell subclass in app/view/table_cells"
    def ui_table_view_cell(name)
      @name = name
      @class_name = "%sCell" % @name.camelize
      template "ui_table_view_cell.rb.erb",
        "app/views/table_cells/#{name.underscorize}_cell.rb"
    end

    option :name
    option :parent_class, {
      desc: "The name of the class to inherit from",
      aliases: ["-p", "--parent"],
      default: "UICollectionViewCell",
      type: :string,
    }
    desc "ui_collection_view_cell", "Creates a UICollectionViewCell subclass in app/view/collection_cells"
    def ui_collection_view_cell(name)
      @name = name
      @class_name = "%sCell" % @name.camelize
      template "ui_table_view_cell.rb.erb",
        "app/views/table_cells/#{name.underscore}_cell.rb"
    end

    private


    def parent_class_name
      options[:parent_class].camelize
    end

    def class_name
      @class_name
    end

    def singular_name
      @name.to_s.underscore.singularize
    end

    def plural_name
      @name.to_s.underscore.pluralize
    end

  end

end
